@@ -179,19 +179,13 @@ implicit_schema <- function(.data) {
       new_fields <- c(left_fields, right_fields)
     }
   } else {
-    # The output schema is based on the aggregations and any group_by vars
-    new_fields <- map(summarize_projection(.data), ~ .$type(old_schm))
-    # * Put group_by_vars first (this can't be done by summarize,
-    #   they have to be last per the aggregate node signature,
-    #   and they get projected to this order after aggregation)
-    # * Infer the output types from the aggregations
-    group_fields <- new_fields[.data$group_by_vars]
     hash <- length(.data$group_by_vars) > 0
-    agg_fields <- imap(
-      new_fields[setdiff(names(new_fields), .data$group_by_vars)],
-      ~ agg_fun_output_type(.data$aggregations[[.y]][["fun"]], .x, hash)
+    # The output schema is based on the aggregations and any group_by vars.
+    # The group_by vars come first.
+    new_fields <- c(
+      group_types(.data, old_schm),
+      aggregate_types(.data, hash, old_schm)
     )
-    new_fields <- c(group_fields, agg_fields)
   }
   schema(!!!new_fields)
 }
