@@ -229,9 +229,11 @@ mmap_open <- function(path, mode = c("read", "write", "readwrite")) {
 #' Handle a range of possible input sources
 #' @param file A character file name, `raw` vector, or an Arrow input stream
 #' @param mmap Logical: whether to memory-map the file (default `TRUE`)
+#' @param random_access Logical: whether the result must be a RandomAccessFile
 #' @return An `InputStream` or a subclass of one.
 #' @keywords internal
-make_readable_file <- function(file, mmap = TRUE) {
+#' @importFrom utils download.file
+make_readable_file <- function(file, mmap = TRUE, random_access = TRUE) {
   if (inherits(file, "SubTreeFileSystem")) {
     filesystem <- file$base_fs
     # SubTreeFileSystem adds a slash to base_path, but filesystems will reject
@@ -239,6 +241,14 @@ make_readable_file <- function(file, mmap = TRUE) {
     path <- sub("/$", "", file$base_path)
     file <- filesystem$OpenInputFile(path)
   } else if (is.string(file)) {
+    # if this is a HTTP URL, we need a local copy to pass to FileSystem$from_uri
+    if (random_access && is_http_url(file)) {
+      tf <- tempfile()
+      download.file(file, tf, quiet = TRUE, mode = "wb")
+      file <- tf
+      on.exit(unlink(tf))
+    }
+
     if (is_url(file)) {
       file <- tryCatch(
         {
