@@ -30,6 +30,7 @@
 #include "arrow/util/decimal.h"
 #include "arrow/util/int_util_overflow.h"
 #include "arrow/util/logging.h"
+#include "arrow/util/ree_util.h"
 #include "arrow/util/utf8.h"
 #include "arrow/visit_data_inline.h"
 #include "arrow/visit_type_inline.h"
@@ -413,6 +414,20 @@ struct ValidateArrayImpl {
     return Status::OK();
   }
 
+  Status Visit(const RunEndEncodedType& type) {
+    switch (type.run_end_type()->id()) {
+      case Type::INT16:
+        return ValidateRunEndEncoded<int16_t>(type);
+      case Type::INT32:
+        return ValidateRunEndEncoded<int32_t>(type);
+      case Type::INT64:
+        return ValidateRunEndEncoded<int64_t>(type);
+      default:
+        return Status::Invalid("Run end type must be int16, int32 or int64, but got: ",
+                               type.run_end_type()->ToString());
+    }
+  }
+
   Status Visit(const ExtensionType& type) {
     // Visit storage
     return ValidateWithType(*type.storage_type());
@@ -622,6 +637,68 @@ struct ValidateArrayImpl {
     return Status::OK();
   }
 
+  template <typename RunEndCType>
+  Status ValidateRunEndEncoded(const RunEndEncodedType& type) {
+    if (data.child_data.size() != 2) {
+      return Status::Invalid(
+          "Run end encoded array should have 2 children; this array has ",
+          data.child_data.size());
+    }
+
+    if (data.buffers.size() > 0 && data.buffers[0] != nullptr) {
+      return Status::Invalid("Run end encoded array should not have a null bitmap.");
+    }
+
+    const auto& run_ends_data = data.child_data[0];
+    const auto& values_data = data.child_data[1];
+
+    if (!run_ends_data) {
+      return Status::Invalid("Run ends array is null pointer");
+    }
+    if (!values_data) {
+      return Status::Invalid("Values array is null pointer");
+    }
+    // We must validate child array buffers are valid before making additional checks.
+    const Status run_ends_valid = RecurseInto(*run_ends_data);
+    if (!run_ends_valid.ok()) {
+      return Status::Invalid("Run ends array invalid: ", run_ends_valid.message());
+    }
+    const Status values_valid = RecurseInto(*values_data);
+    if (!values_valid.ok()) {
+      return Status::Invalid("Values array invalid: ", values_valid.message());
+    }
+
+    RETURN_NOT_OK(ree_util::ValidateRunEndEncodedChildren(
+        type, data.length, run_ends_data, values_data, data.GetNullCount(), data.offset));
+
+    if (run_ends_data->length == 0) {
+      return Status::OK();
+    }
+
+    if (full_validation) {
+      ArraySpan span(data);
+      const auto* run_ends = ree_util::RunEnds<RunEndCType>(span);
+      const int64_t run_ends_length = ree_util::RunEndsArray(span).length;
+      if (run_ends[0] < 1) {
+        return Status::Invalid(
+            "All run ends must be greater than 0 but the first run end is ", run_ends[0]);
+      }
+      int64_t last_run_end = run_ends[0];
+      for (int64_t index = 1; index < run_ends_length; index++) {
+        const int64_t run_end = run_ends[index];
+        if (run_end <= last_run_end) {
+          return Status::Invalid(
+              "Every run end must be strictly greater than the previous run end, "
+              "but run_ends[",
+              index, "] is ", run_end, " and run_ends[", index - 1, "] is ",
+              last_run_end);
+        }
+        last_run_end = run_end;
+      }
+    }
+    return Status::OK();
+  }
+
   template <typename TypeClass>
   Status ValidateOffsets(const TypeClass& type, int64_t offset_limit) {
     using offset_type = typename TypeClass::offset_type;
