@@ -28,10 +28,11 @@
 #include <variant>
 #include <vector>
 
+#include "arrow/acero/aggregate_node.h"
+#include "arrow/acero/exec_plan.h"
+#include "arrow/acero/options.h"
 #include "arrow/compute/api_aggregate.h"
-#include "arrow/compute/exec/exec_plan.h"
-#include "arrow/compute/exec/expression.h"
-#include "arrow/compute/exec/options.h"
+#include "arrow/compute/expression.h"
 #include "arrow/compute/kernel.h"
 #include "arrow/dataset/dataset.h"
 #include "arrow/dataset/discovery.h"
@@ -92,6 +93,42 @@ Result<EmitInfo> GetEmitInfo(const RelMessage& rel,
   return std::move(emit_info);
 }
 
+Result<DeclarationInfo> ProcessEmitProject(
+    std::optional<substrait::RelCommon> rel_common_opt,
+    const DeclarationInfo& project_declr, const std::shared_ptr<Schema>& input_schema) {
+  if (rel_common_opt) {
+    switch (rel_common_opt->emit_kind_case()) {
+      case substrait::RelCommon::EmitKindCase::kDirect:
+        return project_declr;
+      case substrait::RelCommon::EmitKindCase::kEmit: {
+        const auto& emit = rel_common_opt->emit();
+        int emit_size = emit.output_mapping_size();
+        const auto& proj_options = checked_cast<const acero::ProjectNodeOptions&>(
+            *project_declr.declaration.options);
+        FieldVector emit_fields(emit_size);
+        std::vector<compute::Expression> emit_proj_exprs(emit_size);
+        for (int i = 0; i < emit_size; i++) {
+          int32_t map_id = emit.output_mapping(i);
+          emit_fields[i] = input_schema->field(map_id);
+          emit_proj_exprs[i] = std::move(proj_options.expressions[map_id]);
+        }
+        // Note: DeclarationInfo is created by considering the input to the
+        // ProjectRel and the ProjectNodeOptions are set by only considering
+        // what is in the emit expression in Substrait.
+        return DeclarationInfo{
+            acero::Declaration::Sequence(
+                {std::get<acero::Declaration>(project_declr.declaration.inputs[0]),
+                 {"project", acero::ProjectNodeOptions{std::move(emit_proj_exprs)}}}),
+            schema(std::move(emit_fields))};
+      }
+      default:
+        return Status::Invalid("Invalid emit case");
+    }
+  } else {
+    return project_declr;
+  }
+}
+
 template <typename RelMessage>
 Result<DeclarationInfo> ProcessEmit(const RelMessage& rel,
                                     const DeclarationInfo& no_emit_declr,
@@ -103,10 +140,9 @@ Result<DeclarationInfo> ProcessEmit(const RelMessage& rel,
       case substrait::RelCommon::EmitKindCase::kEmit: {
         ARROW_ASSIGN_OR_RAISE(auto emit_info, GetEmitInfo(rel, schema));
         return DeclarationInfo{
-            compute::Declaration::Sequence(
-                {no_emit_declr.declaration,
-                 {"project",
-                  compute::ProjectNodeOptions{std::move(emit_info.expressions)}}}),
+            acero::Declaration::Sequence({no_emit_declr.declaration,
+                                          {"project", acero::ProjectNodeOptions{std::move(
+                                                          emit_info.expressions)}}}),
             std::move(emit_info.schema)};
       }
       default:
@@ -120,38 +156,104 @@ Result<DeclarationInfo> ProcessEmit(const RelMessage& rel,
 /// get the Acero relation with or without emit.
 template <>
 Result<DeclarationInfo> ProcessEmit(const substrait::ProjectRel& rel,
-                                    const DeclarationInfo& project_declr,
-                                    const std::shared_ptr<Schema>& input_schema) {
-  if (rel.has_common()) {
-    switch (rel.common().emit_kind_case()) {
-      case substrait::RelCommon::EmitKindCase::kDirect:
-        return project_declr;
-      case substrait::RelCommon::EmitKindCase::kEmit: {
-        const auto& emit = rel.common().emit();
-        int emit_size = emit.output_mapping_size();
-        const auto& proj_options = checked_cast<const compute::ProjectNodeOptions&>(
-            *project_declr.declaration.options);
-        FieldVector emit_fields(emit_size);
-        std::vector<compute::Expression> emit_proj_exprs(emit_size);
-        for (int i = 0; i < emit_size; i++) {
-          int32_t map_id = emit.output_mapping(i);
-          emit_fields[i] = input_schema->field(map_id);
-          emit_proj_exprs[i] = std::move(proj_options.expressions[map_id]);
-        }
-        // Note: DeclarationInfo is created by considering the input to the
-        // ProjectRel and the ProjectNodeOptions are set by only considering
-        // what is in the emit expression in Substrait.
-        return DeclarationInfo{
-            compute::Declaration::Sequence(
-                {std::get<compute::Declaration>(project_declr.declaration.inputs[0]),
-                 {"project", compute::ProjectNodeOptions{std::move(emit_proj_exprs)}}}),
-            schema(std::move(emit_fields))};
+                                    const DeclarationInfo& no_emit_declr,
+                                    const std::shared_ptr<Schema>& schema) {
+  return ProcessEmitProject(rel.has_common() ? std::optional(rel.common()) : std::nullopt,
+                            no_emit_declr, schema);
+}
+
+Result<DeclarationInfo> ProcessExtensionEmit(
+    const DeclarationInfo& no_emit_declr, const std::vector<int>& emit_order,
+    const std::vector<int>& field_output_indices) {
+  const std::shared_ptr<Schema>& input_schema = no_emit_declr.output_schema;
+  std::vector<compute::Expression> proj_field_refs;
+  proj_field_refs.reserve(emit_order.size());
+  FieldVector emit_fields;
+  emit_fields.reserve(emit_order.size());
+
+  for (int emit_idx : emit_order) {
+    if (emit_idx < 0 || static_cast<size_t>(emit_idx) >= field_output_indices.size()) {
+      return Status::Invalid("Out of bounds emit index ", emit_idx);
+    }
+    int field_idx = field_output_indices[emit_idx];
+    if (field_idx < 0) {
+      return Status::Invalid("Non-output emit index ", emit_idx);
+    }
+    proj_field_refs.push_back(compute::field_ref(FieldRef(field_idx)));
+    emit_fields.push_back(input_schema->field(field_idx));
+  }
+
+  std::shared_ptr<Schema> emit_schema = schema(std::move(emit_fields));
+
+  return DeclarationInfo{
+      acero::Declaration::Sequence(
+          {no_emit_declr.declaration,
+           {"project", acero::ProjectNodeOptions{std::move(proj_field_refs)}}}),
+      std::move(emit_schema)};
+}
+
+Result<RelationInfo> GetExtensionRelationInfo(const substrait::Rel& rel,
+                                              const ExtensionSet& ext_set,
+                                              const ConversionOptions& conv_opts,
+                                              std::vector<DeclarationInfo>* inputs_arg) {
+  if (inputs_arg == nullptr) {
+    std::vector<DeclarationInfo> inputs_tmp;
+    return GetExtensionRelationInfo(rel, ext_set, conv_opts, &inputs_tmp);
+  }
+  std::vector<DeclarationInfo>& inputs = *inputs_arg;
+  inputs.clear();
+  switch (rel.rel_type_case()) {
+    case substrait::Rel::RelTypeCase::kExtensionLeaf: {
+      const auto& ext = rel.extension_leaf();
+      DefaultExtensionDetails detail{ext.detail()};
+      return conv_opts.extension_provider->MakeRel(conv_opts, inputs, detail, ext_set);
+    }
+
+    case substrait::Rel::RelTypeCase::kExtensionSingle: {
+      const auto& ext = rel.extension_single();
+      ARROW_ASSIGN_OR_RAISE(DeclarationInfo input_info,
+                            FromProto(ext.input(), ext_set, conv_opts));
+      inputs.push_back(std::move(input_info));
+      DefaultExtensionDetails detail{ext.detail()};
+      return conv_opts.extension_provider->MakeRel(conv_opts, inputs, detail, ext_set);
+    }
+
+    case substrait::Rel::RelTypeCase::kExtensionMulti: {
+      const auto& ext = rel.extension_multi();
+      for (const auto& input : ext.inputs()) {
+        ARROW_ASSIGN_OR_RAISE(auto input_info, FromProto(input, ext_set, conv_opts));
+        inputs.push_back(std::move(input_info));
       }
-      default:
-        return Status::Invalid("Invalid emit case");
+      DefaultExtensionDetails detail{ext.detail()};
+      return conv_opts.extension_provider->MakeRel(conv_opts, inputs, detail, ext_set);
+    }
+
+    default: {
+      return Status::Invalid("Invalid extension relation case ", rel.rel_type_case());
+    }
+  }
+}
+
+std::optional<substrait::RelCommon> GetExtensionRelCommon(const substrait::Rel& rel) {
+  switch (rel.rel_type_case()) {
+    case substrait::Rel::RelTypeCase::kExtensionLeaf: {
+      const auto& ext = rel.extension_leaf();
+      return ext.has_common() ? std::optional(ext.common()) : std::nullopt;
+    }
+
+    case substrait::Rel::RelTypeCase::kExtensionSingle: {
+      const auto& ext = rel.extension_single();
+      return ext.has_common() ? std::optional(ext.common()) : std::nullopt;
+    }
+
+    case substrait::Rel::RelTypeCase::kExtensionMulti: {
+      const auto& ext = rel.extension_multi();
+      return ext.has_common() ? std::optional(ext.common()) : std::nullopt;
+    }
+
+    default: {
+      return std::nullopt;
     }
-  } else {
-    return project_declr;
   }
 }
 
@@ -191,6 +293,47 @@ Status DiscoverFilesFromDir(const std::shared_ptr<fs::LocalFileSystem>& local_fs
   return Status::OK();
 }
 
+namespace internal {
+
+Result<compute::Aggregate> ParseAggregateMeasure(
+    const substrait::AggregateRel::Measure& agg_measure, const ExtensionSet& ext_set,
+    const ConversionOptions& conversion_options, bool is_hash,
+    const std::shared_ptr<Schema> input_schema) {
+  if (agg_measure.has_measure()) {
+    if (agg_measure.has_filter()) {
+      return Status::NotImplemented("Aggregate filters are not supported.");
+    }
+    const auto& agg_func = agg_measure.measure();
+    ARROW_ASSIGN_OR_RAISE(SubstraitCall aggregate_call,
+                          FromProto(agg_func, is_hash, ext_set, conversion_options));
+    ExtensionIdRegistry::SubstraitAggregateToArrow converter;
+    if (aggregate_call.id().uri.empty() || aggregate_call.id().uri[0] == '/') {
+      ARROW_ASSIGN_OR_RAISE(converter,
+                            ext_set.registry()->GetSubstraitAggregateToArrowFallback(
+                                aggregate_call.id().name));
+    } else {
+      ARROW_ASSIGN_OR_RAISE(converter, ext_set.registry()->GetSubstraitAggregateToArrow(
+                                           aggregate_call.id()));
+    }
+    return converter(aggregate_call);
+  } else {
+    return Status::Invalid("substrait::AggregateFunction not provided");
+  }
+}
+
+ARROW_ENGINE_EXPORT Result<DeclarationInfo> MakeAggregateDeclaration(
+    acero::Declaration input_decl, std::shared_ptr<Schema> aggregate_schema,
+    std::vector<compute::Aggregate> aggregates, std::vector<FieldRef> keys,
+    std::vector<FieldRef> segment_keys) {
+  return DeclarationInfo{
+      acero::Declaration::Sequence(
+          {std::move(input_decl),
+           {"aggregate", acero::AggregateNodeOptions{aggregates, keys, segment_keys}}}),
+      aggregate_schema};
+}
+
+}  // namespace internal
+
 Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet& ext_set,
                                   const ConversionOptions& conversion_options) {
   static bool dataset_init = false;
@@ -236,16 +379,15 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
         const substrait::ReadRel::NamedTable& named_table = read.named_table();
         std::vector<std::string> table_names(named_table.names().begin(),
                                              named_table.names().end());
-        ARROW_ASSIGN_OR_RAISE(compute::Declaration source_decl,
-                              named_table_provider(table_names));
+        ARROW_ASSIGN_OR_RAISE(acero::Declaration source_decl,
+                              named_table_provider(table_names, *base_schema));
 
         if (!source_decl.IsValid()) {
           return Status::Invalid("Invalid NamedTable Source");
         }
 
-        return ProcessEmit(std::move(read),
-                           DeclarationInfo{std::move(source_decl), base_schema},
-                           std::move(base_schema));
+        return ProcessEmit(read, DeclarationInfo{std::move(source_decl), base_schema},
+                           base_schema);
       }
 
       if (!read.has_local_files()) {
@@ -388,11 +530,10 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
       ARROW_ASSIGN_OR_RAISE(auto ds, ds_factory->Finish(base_schema));
 
       DeclarationInfo scan_declaration{
-          compute::Declaration{"scan", dataset::ScanNodeOptions{ds, scan_options}},
+          acero::Declaration{"scan", dataset::ScanNodeOptions{ds, scan_options}},
           base_schema};
 
-      return ProcessEmit(std::move(read), std::move(scan_declaration),
-                         std::move(base_schema));
+      return ProcessEmit(read, scan_declaration, base_schema);
     }
 
     case substrait::Rel::RelTypeCase::kFilter: {
@@ -411,14 +552,13 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
       ARROW_ASSIGN_OR_RAISE(auto condition,
                             FromProto(filter.condition(), ext_set, conversion_options));
       DeclarationInfo filter_declaration{
-          compute::Declaration::Sequence({
+          acero::Declaration::Sequence({
               std::move(input.declaration),
-              {"filter", compute::FilterNodeOptions{std::move(condition)}},
+              {"filter", acero::FilterNodeOptions{std::move(condition)}},
           }),
           input.output_schema};
 
-      return ProcessEmit(std::move(filter), std::move(filter_declaration),
-                         input.output_schema);
+      return ProcessEmit(filter, filter_declaration, input.output_schema);
     }
 
     case substrait::Rel::RelTypeCase::kProject: {
@@ -445,8 +585,9 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
         std::shared_ptr<Field> project_field;
         ARROW_ASSIGN_OR_RAISE(compute::Expression des_expr,
                               FromProto(expr, ext_set, conversion_options));
-        auto bound_expr = des_expr.Bind(*input.output_schema);
-        if (auto* expr_call = bound_expr->call()) {
+        ARROW_ASSIGN_OR_RAISE(compute::Expression bound_expr,
+                              des_expr.Bind(*input.output_schema));
+        if (auto* expr_call = bound_expr.call()) {
           project_field = field(expr_call->function_name,
                                 expr_call->kernel->signature->out_type().type());
         } else if (auto* field_ref = des_expr.field_ref()) {
@@ -464,14 +605,13 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
       }
 
       DeclarationInfo project_declaration{
-          compute::Declaration::Sequence({
+          acero::Declaration::Sequence({
               std::move(input.declaration),
-              {"project", compute::ProjectNodeOptions{std::move(expressions)}},
+              {"project", acero::ProjectNodeOptions{std::move(expressions)}},
           }),
           project_schema};
 
-      return ProcessEmit(std::move(project), std::move(project_declaration),
-                         std::move(project_schema));
+      return ProcessEmit(project, project_declaration, project_schema);
     }
 
     case substrait::Rel::RelTypeCase::kJoin: {
@@ -486,27 +626,27 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
         return Status::Invalid("substrait::JoinRel with no right relation");
       }
 
-      compute::JoinType join_type;
+      acero::JoinType join_type;
       switch (join.type()) {
         case substrait::JoinRel::JOIN_TYPE_UNSPECIFIED:
           return Status::NotImplemented("Unspecified join type is not supported");
         case substrait::JoinRel::JOIN_TYPE_INNER:
-          join_type = compute::JoinType::INNER;
+          join_type = acero::JoinType::INNER;
           break;
         case substrait::JoinRel::JOIN_TYPE_OUTER:
-          join_type = compute::JoinType::FULL_OUTER;
+          join_type = acero::JoinType::FULL_OUTER;
           break;
         case substrait::JoinRel::JOIN_TYPE_LEFT:
-          join_type = compute::JoinType::LEFT_OUTER;
+          join_type = acero::JoinType::LEFT_OUTER;
           break;
         case substrait::JoinRel::JOIN_TYPE_RIGHT:
-          join_type = compute::JoinType::RIGHT_OUTER;
+          join_type = acero::JoinType::RIGHT_OUTER;
           break;
         case substrait::JoinRel::JOIN_TYPE_SEMI:
-          join_type = compute::JoinType::LEFT_SEMI;
+          join_type = acero::JoinType::LEFT_SEMI;
           break;
         case substrait::JoinRel::JOIN_TYPE_ANTI:
-          join_type = compute::JoinType::LEFT_ANTI;
+          join_type = acero::JoinType::LEFT_ANTI;
           break;
         default:
           return Status::Invalid("Unsupported join type");
@@ -531,11 +671,11 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
             expression.ToString());
       }
 
-      compute::JoinKeyCmp join_key_cmp;
+      acero::JoinKeyCmp join_key_cmp;
       if (callptr->function_name == "equal") {
-        join_key_cmp = compute::JoinKeyCmp::EQ;
+        join_key_cmp = acero::JoinKeyCmp::EQ;
       } else if (callptr->function_name == "is_not_distinct_from") {
-        join_key_cmp = compute::JoinKeyCmp::IS;
+        join_key_cmp = acero::JoinKeyCmp::IS;
       } else {
         return Status::Invalid(
             "Only `equal` or `is_not_distinct_from` are supported for join key "
@@ -567,18 +707,17 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
       std::vector<int> adjusted_field_indices(right_field_path->indices());
       adjusted_field_indices[0] -= num_left_fields;
       FieldPath adjusted_right_keys(adjusted_field_indices);
-      compute::HashJoinNodeOptions join_options{{std::move(*left_keys)},
-                                                {std::move(adjusted_right_keys)}};
+      acero::HashJoinNodeOptions join_options{{std::move(*left_keys)},
+                                              {std::move(adjusted_right_keys)}};
       join_options.join_type = join_type;
       join_options.key_cmp = {join_key_cmp};
-      compute::Declaration join_dec{"hashjoin", std::move(join_options)};
+      acero::Declaration join_dec{"hashjoin", std::move(join_options)};
       join_dec.inputs.emplace_back(std::move(left.declaration));
       join_dec.inputs.emplace_back(std::move(right.declaration));
 
       DeclarationInfo join_declaration{std::move(join_dec), join_schema};
 
-      return ProcessEmit(std::move(join), std::move(join_declaration),
-                         std::move(join_schema));
+      return ProcessEmit(join, join_declaration, join_schema);
     }
     case substrait::Rel::RelTypeCase::kAggregate: {
       const auto& aggregate = rel.aggregate();
@@ -599,22 +738,17 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
 
       // prepare output schema from aggregates
       auto input_schema = input.output_schema;
-      // store key fields to be used when output schema is created
-      std::vector<int> key_field_ids;
       std::vector<FieldRef> keys;
       if (aggregate.groupings_size() > 0) {
         const substrait::AggregateRel::Grouping& group = aggregate.groupings(0);
         int grouping_expr_size = group.grouping_expressions_size();
         keys.reserve(grouping_expr_size);
-        key_field_ids.reserve(grouping_expr_size);
         for (int exp_id = 0; exp_id < grouping_expr_size; exp_id++) {
           ARROW_ASSIGN_OR_RAISE(
               compute::Expression expr,
               FromProto(group.grouping_expressions(exp_id), ext_set, conversion_options));
           const FieldRef* field_ref = expr.field_ref();
           if (field_ref) {
-            ARROW_ASSIGN_OR_RAISE(auto match, field_ref->FindOne(*input_schema));
-            key_field_ids.emplace_back(std::move(match[0]));
             keys.emplace_back(std::move(*field_ref));
           } else {
             return Status::Invalid(
@@ -623,66 +757,77 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
         }
       }
 
-      int measure_size = aggregate.measures_size();
+      const int measure_size = aggregate.measures_size();
       std::vector<compute::Aggregate> aggregates;
       aggregates.reserve(measure_size);
-      // store aggregate fields to be used when output schema is created
-      std::vector<int> agg_src_field_ids(measure_size);
       for (int measure_id = 0; measure_id < measure_size; measure_id++) {
         const auto& agg_measure = aggregate.measures(measure_id);
-        if (agg_measure.has_measure()) {
-          if (agg_measure.has_filter()) {
-            return Status::NotImplemented("Aggregate filters are not supported.");
-          }
-          const auto& agg_func = agg_measure.measure();
-          ARROW_ASSIGN_OR_RAISE(
-              SubstraitCall aggregate_call,
-              FromProto(agg_func, !keys.empty(), ext_set, conversion_options));
-          ExtensionIdRegistry::SubstraitAggregateToArrow converter;
-          if (aggregate_call.id().uri.empty() || aggregate_call.id().uri[0] == '/') {
-            ARROW_ASSIGN_OR_RAISE(
-                converter, ext_set.registry()->GetSubstraitAggregateToArrowFallback(
-                               aggregate_call.id().name));
-          } else {
-            ARROW_ASSIGN_OR_RAISE(
-                converter,
-                ext_set.registry()->GetSubstraitAggregateToArrow(aggregate_call.id()));
-          }
-          ARROW_ASSIGN_OR_RAISE(compute::Aggregate arrow_agg, converter(aggregate_call));
-
-          // find aggregate field ids from schema
-          const auto field_ref = arrow_agg.target;
-          ARROW_ASSIGN_OR_RAISE(auto match, field_ref.FindOne(*input_schema));
-          agg_src_field_ids[measure_id] = match[0];
-
-          aggregates.push_back(std::move(arrow_agg));
-        } else {
-          return Status::Invalid("substrait::AggregateFunction not provided");
-        }
-      }
-      FieldVector output_fields;
-      output_fields.reserve(key_field_ids.size() + agg_src_field_ids.size());
-      // extract aggregate fields to output schema
-      for (int id = 0; id < static_cast<int>(agg_src_field_ids.size()); id++) {
-        output_fields.emplace_back(input_schema->field(agg_src_field_ids[id]));
-      }
-      // extract key fields to output schema
-      for (int id = 0; id < static_cast<int>(key_field_ids.size()); id++) {
-        output_fields.emplace_back(input_schema->field(key_field_ids[id]));
+        ARROW_ASSIGN_OR_RAISE(
+            auto aggregate,
+            internal::ParseAggregateMeasure(agg_measure, ext_set, conversion_options,
+                                            /*is_hash=*/!keys.empty(), input_schema));
+        aggregates.push_back(std::move(aggregate));
       }
 
-      std::shared_ptr<Schema> aggregate_schema = schema(std::move(output_fields));
+      ARROW_ASSIGN_OR_RAISE(auto aggregate_schema,
+                            acero::aggregate::MakeOutputSchema(
+                                input_schema, keys, /*segment_keys=*/{}, aggregates));
 
-      DeclarationInfo aggregate_declaration{
-          compute::Declaration::Sequence(
-              {std::move(input.declaration),
-               {"aggregate", compute::AggregateNodeOptions{aggregates, keys}}}),
-          aggregate_schema};
+      ARROW_ASSIGN_OR_RAISE(
+          auto aggregate_declaration,
+          internal::MakeAggregateDeclaration(std::move(input.declaration),
+                                             aggregate_schema, std::move(aggregates),
+                                             std::move(keys), /*segment_keys=*/{}));
 
       return ProcessEmit(std::move(aggregate), std::move(aggregate_declaration),
                          std::move(aggregate_schema));
     }
 
+    case substrait::Rel::RelTypeCase::kExtensionLeaf:
+    case substrait::Rel::RelTypeCase::kExtensionSingle:
+    case substrait::Rel::RelTypeCase::kExtensionMulti: {
+      std::vector<DeclarationInfo> ext_rel_inputs;
+      ARROW_ASSIGN_OR_RAISE(
+          auto ext_rel_info,
+          GetExtensionRelationInfo(rel, ext_set, conversion_options, &ext_rel_inputs));
+      const auto& ext_decl_info = ext_rel_info.decl_info;
+      auto ext_common_opt = GetExtensionRelCommon(rel);
+      bool has_emit = ext_common_opt && ext_common_opt->emit_kind_case() ==
+                                            substrait::RelCommon::EmitKindCase::kEmit;
+      if (!ext_rel_info.field_output_indices) {
+        if (!has_emit) {
+          return ext_decl_info;
+        }
+        return Status::NotImplemented("Emit not supported by ",
+                                      ext_decl_info.declaration.factory_name);
+      }
+      // Set up the emit order - an ordered list of indices that specifies an output
+      // mapping as expected by Substrait. This is a sublist of [0..N), where N is the
+      // total number of input fields across all inputs of the relation, that selects
+      // from these input fields.
+      std::vector<int> emit_order;
+      if (has_emit) {
+        // the emit order is defined in the Substrait plan - pick it up
+        const auto& emit_info = ext_common_opt->emit();
+        emit_order.reserve(emit_info.output_mapping_size());
+        for (const auto& emit_idx : emit_info.output_mapping()) {
+          emit_order.push_back(emit_idx);
+        }
+      } else {
+        // the emit order is the default output mapping [0..N)
+        int emit_size = 0;
+        for (const auto& input : ext_rel_inputs) {
+          emit_size += input.output_schema->num_fields();
+        }
+        emit_order.reserve(emit_size);
+        for (int emit_idx = 0; emit_idx < emit_size; emit_idx++) {
+          emit_order.push_back(emit_idx);
+        }
+      }
+      return ProcessExtensionEmit(ext_decl_info, emit_order,
+                                  *ext_rel_info.field_output_indices);
+    }
+
     case substrait::Rel::RelTypeCase::kSet: {
       const auto& set = rel.set();
       RETURN_NOT_OK(CheckRelCommon(set, conversion_options));
@@ -710,7 +855,7 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
           return Status::Invalid("Unknown union type");
       }
       int input_size = set.inputs_size();
-      compute::Declaration union_declr{"union", compute::ExecNodeOptions{}};
+      acero::Declaration union_declr{"union", acero::ExecNodeOptions{}};
       std::shared_ptr<Schema> union_schema;
       for (int input_id = 0; input_id < input_size; input_id++) {
         ARROW_ASSIGN_OR_RAISE(
@@ -722,37 +867,7 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
       }
 
       auto set_declaration = DeclarationInfo{union_declr, union_schema};
-      return ProcessEmit(std::move(set), std::move(set_declaration),
-                         std::move(union_schema));
-    }
-    case substrait::Rel::RelTypeCase::kExtensionLeaf: {
-      const auto& ext = rel.extension_leaf();
-      ARROW_ASSIGN_OR_RAISE(
-          auto ext_leaf_decl,
-          conversion_options.extension_provider->MakeRel({}, ext.detail(), ext_set));
-      return ProcessEmit(ext, std::move(ext_leaf_decl), ext_leaf_decl.output_schema);
-    }
-    case substrait::Rel::RelTypeCase::kExtensionSingle: {
-      const auto& ext = rel.extension_single();
-      ARROW_ASSIGN_OR_RAISE(DeclarationInfo input,
-                            FromProto(ext.input(), ext_set, conversion_options));
-      ARROW_ASSIGN_OR_RAISE(
-          auto ext_single_decl,
-          conversion_options.extension_provider->MakeRel({input}, ext.detail(), ext_set));
-      return ProcessEmit(ext, std::move(ext_single_decl), ext_single_decl.output_schema);
-    }
-    case substrait::Rel::RelTypeCase::kExtensionMulti: {
-      const auto& ext = rel.extension_multi();
-      std::vector<DeclarationInfo> inputs;
-      for (const auto& input : ext.inputs()) {
-        ARROW_ASSIGN_OR_RAISE(auto input_info,
-                              FromProto(input, ext_set, conversion_options));
-        inputs.push_back(std::move(input_info));
-      }
-      ARROW_ASSIGN_OR_RAISE(
-          auto ext_multi_decl,
-          conversion_options.extension_provider->MakeRel(inputs, ext.detail(), ext_set));
-      return ProcessEmit(ext, std::move(ext_multi_decl), ext_multi_decl.output_schema);
+      return ProcessEmit(set, set_declaration, union_schema);
     }
 
     default:
@@ -760,23 +875,22 @@ Result<DeclarationInfo> FromProto(const substrait::Rel& rel, const ExtensionSet&
   }
 
   return Status::NotImplemented(
-      "conversion to arrow::compute::Declaration from Substrait relation ",
+      "conversion to arrow::acero::Declaration from Substrait relation ",
       rel.DebugString());
 }
 
 namespace {
 
-Result<std::shared_ptr<Schema>> ExtractSchemaToBind(const compute::Declaration& declr) {
+Result<std::shared_ptr<Schema>> ExtractSchemaToBind(const acero::Declaration& declr) {
   std::shared_ptr<Schema> bind_schema;
   if (declr.factory_name == "scan") {
     const auto& opts = checked_cast<const dataset::ScanNodeOptions&>(*(declr.options));
     bind_schema = opts.dataset->schema();
   } else if (declr.factory_name == "filter") {
-    auto input_declr = std::get<compute::Declaration>(declr.inputs[0]);
+    auto input_declr = std::get<acero::Declaration>(declr.inputs[0]);
     ARROW_ASSIGN_OR_RAISE(bind_schema, ExtractSchemaToBind(input_declr));
   } else if (declr.factory_name == "named_table") {
-    const auto& opts =
-        checked_cast<const compute::NamedTableNodeOptions&>(*declr.options);
+    const auto& opts = checked_cast<const acero::NamedTableNodeOptions&>(*declr.options);
     bind_schema = opts.schema;
   } else if (declr.factory_name == "sink") {
     // Note that the sink has no output_schema
@@ -789,11 +903,11 @@ Result<std::shared_ptr<Schema>> ExtractSchemaToBind(const compute::Declaration&
 }
 
 Result<std::unique_ptr<substrait::ReadRel>> NamedTableRelationConverter(
-    const std::shared_ptr<Schema>& schema, const compute::Declaration& declaration,
+    const std::shared_ptr<Schema>& schema, const acero::Declaration& declaration,
     ExtensionSet* ext_set, const ConversionOptions& conversion_options) {
   auto read_rel = std::make_unique<substrait::ReadRel>();
   const auto& named_table_options =
-      checked_cast<const compute::NamedTableNodeOptions&>(*declaration.options);
+      checked_cast<const acero::NamedTableNodeOptions&>(*declaration.options);
 
   // set schema
   ARROW_ASSIGN_OR_RAISE(auto named_struct, ToProto(*schema, ext_set, conversion_options));
@@ -813,7 +927,7 @@ Result<std::unique_ptr<substrait::ReadRel>> NamedTableRelationConverter(
 }
 
 Result<std::unique_ptr<substrait::ReadRel>> ScanRelationConverter(
-    const std::shared_ptr<Schema>& schema, const compute::Declaration& declaration,
+    const std::shared_ptr<Schema>& schema, const acero::Declaration& declaration,
     ExtensionSet* ext_set, const ConversionOptions& conversion_options) {
   auto read_rel = std::make_unique<substrait::ReadRel>();
   const auto& scan_node_options =
@@ -857,11 +971,11 @@ Result<std::unique_ptr<substrait::ReadRel>> ScanRelationConverter(
 }
 
 Result<std::unique_ptr<substrait::FilterRel>> FilterRelationConverter(
-    const std::shared_ptr<Schema>& schema, const compute::Declaration& declaration,
+    const std::shared_ptr<Schema>& schema, const acero::Declaration& declaration,
     ExtensionSet* ext_set, const ConversionOptions& conversion_options) {
   auto filter_rel = std::make_unique<substrait::FilterRel>();
   const auto& filter_node_options =
-      checked_cast<const compute::FilterNodeOptions&>(*(declaration.options));
+      checked_cast<const acero::FilterNodeOptions&>(*(declaration.options));
 
   auto filter_expr = filter_node_options.filter_expression;
   compute::Expression bound_expression;
@@ -875,9 +989,8 @@ Result<std::unique_ptr<substrait::FilterRel>> FilterRelationConverter(
 
   // handling input
   auto declr_input = declaration.inputs[0];
-  ARROW_ASSIGN_OR_RAISE(
-      auto input_rel,
-      ToProto(std::get<compute::Declaration>(declr_input), ext_set, conversion_options));
+  ARROW_ASSIGN_OR_RAISE(auto input_rel, ToProto(std::get<acero::Declaration>(declr_input),
+                                                ext_set, conversion_options));
   filter_rel->set_allocated_input(input_rel.release());
 
   ARROW_ASSIGN_OR_RAISE(auto subs_expr,
@@ -888,7 +1001,7 @@ Result<std::unique_ptr<substrait::FilterRel>> FilterRelationConverter(
 
 }  // namespace
 
-Status SerializeAndCombineRelations(const compute::Declaration& declaration,
+Status SerializeAndCombineRelations(const acero::Declaration& declaration,
                                     ExtensionSet* ext_set,
                                     std::unique_ptr<substrait::Rel>* rel,
                                     const ConversionOptions& conversion_options) {
@@ -916,7 +1029,7 @@ Status SerializeAndCombineRelations(const compute::Declaration& declaration,
     // Generally when a plan is deserialized the declaration will be a sink declaration.
     // Since there is no Sink relation in substrait, this function would be recursively
     // called on the input of the Sink declaration.
-    auto sink_input_decl = std::get<compute::Declaration>(declaration.inputs[0]);
+    auto sink_input_decl = std::get<acero::Declaration>(declaration.inputs[0]);
     RETURN_NOT_OK(
         SerializeAndCombineRelations(sink_input_decl, ext_set, rel, conversion_options));
   } else {
@@ -928,7 +1041,7 @@ Status SerializeAndCombineRelations(const compute::Declaration& declaration,
 }
 
 Result<std::unique_ptr<substrait::Rel>> ToProto(
-    const compute::Declaration& declr, ExtensionSet* ext_set,
+    const acero::Declaration& declr, ExtensionSet* ext_set,
     const ConversionOptions& conversion_options) {
   auto rel = std::make_unique<substrait::Rel>();
   RETURN_NOT_OK(SerializeAndCombineRelations(declr, ext_set, &rel, conversion_options));
